/* ignuit - Educational software for the GNOME, following the Leitner
 * flash-card system.
 *
 * Copyright (C) 2008, 2009, Timothy Musson
 *
 * <trmusson@ihug.co.nz>
 * http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <glib.h>
#include <gst/gst.h>


static GMainLoop *loop;
static gboolean lock = FALSE;


/* The functions cb_bus and play_audio are taken pretty much
 * directly from GStreamer Application Development Manual (0.10.20.1)
 */

static gboolean
cb_bus (GstBus *bus, GstMessage *message, gpointer data)
{
    switch (GST_MESSAGE_TYPE(message)) {
    case GST_MESSAGE_ERROR:
        {
        GError *err;
        gchar *debug;

        gst_message_parse_error (message, &err, &debug);
        g_warning ("play_audio: error: %s\n", err->message);
        g_error_free (err);
        g_free (debug);

        g_main_loop_quit (loop);
        break;
        }
    case GST_MESSAGE_EOS:
        /* end-of-stream */
        g_main_loop_quit (loop);
        break;
    default:
        break;
    }

    return TRUE;
}


void
play_audio (const gchar *uri)
{
    GstElement *play;
    GstBus *bus;


    if (lock) { return; }

    lock = TRUE;

    loop = g_main_loop_new (NULL, FALSE);

    play = gst_element_factory_make ("playbin", "play");
    g_object_set (G_OBJECT(play), "uri", uri, NULL);

    bus = gst_pipeline_get_bus (GST_PIPELINE(play));
    gst_bus_add_watch (bus, cb_bus, loop);
    gst_object_unref (bus);

    gst_element_set_state (play, GST_STATE_PLAYING);

    g_main_loop_run (loop);

    gst_element_set_state (play, GST_STATE_NULL);
    gst_object_unref (GST_OBJECT (play));

    lock = FALSE;
}

